#! /usr/bin/perl

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States. Recipients
# of this software assume all responsibilities associated with its operation,
# modification and maintenance. 

#################################################################
# Usage Example: 
# create.msb.pl out/json/cineast_multimediaobject.json out/json/cineast_segment.json ../video_data/video_data.msb

# Input:
# cineast_multimediaobject.json : JSON file that maps each original video file to a randomly selected name ID (generated by the cineast tool)
# cineast_segment.json : The shot segmentation file in JSON format (generated by the cineast tool)

# Output:
# video_data.msb : The master shot boundary table file (plain text) 

################################################################

use JSON qw( decode_json );
use JSON::Parse 'json_file_to_perl';

if (scalar(@ARGV) < 3)
{
    print "Please use the correct parameters to run the script!\n";
    print "Usage: create.msb.pl out/json/cineast_multimediaobject.json out/json/cineast_segment.json ../video_data/video_data.msb\n";
    exit;
}

# take input cineast_multimediaobject.json
$file_ids = $ARGV[0];

# convert json to perl array
my @p = json_file_to_perl ($file_ids);

# take input cineast_segment.json
$segments = $ARGV[1];

# convert json to perl array                                                                                                                          
my @seg = json_file_to_perl ($segments);

# get number of videos from array/json
$num_videos = scalar(@{$p[0]});

# store hash of video names
for ($i = 0; $i<$num_videos; $i++)
{
    $video_hash{$p[0][$i]->{'objectid'}} = $p[0][$i]->{'name'};
}
################################################################

# get number of video segments/shots from array/json                                                                                                 
$num_segments = scalar(@{$seg[0]});

# write out MSB file
$out = $ARGV[2];
open OUT, "> $out" or die "can't open file $!";


for ($i = 0; $i<$num_segments; $i++)
{
    # build shot_id name
    $shot_name = substr($video_hash{$seg[0][$i]->{'objectid'}},0,-4)."_".$seg[0][$i]->{'segmentnumber'};

    if (not exists ($msb{$video_hash{$seg[0][$i]->{'objectid'}}}{$seg[0][$i]->{'segmentstart'}}{$seg[0][$i]->{'segmentend'}}))
	{
	    $msb{$video_hash{$seg[0][$i]->{'objectid'}}}{$seg[0][$i]->{'segmentstart'}}{$seg[0][$i]->{'segmentend'}} = $shot_name;
	    printf OUT "%15s %15s %5d %5d %15.6f %15.6f\n",$seg[0][$i]->{'objectid'},$shot_name,$seg[0][$i]->{'segmentstart'},$seg[0][$i]->{'segmentend'},$seg[0][$i]->{'segmentstartabs'},$seg[0][$i]->{'segmentendabs'};
	}
    else
	{
	    if (-e "out/representative_frames/$seg[0][$i]->{'objectid'}/$seg[0][$i]->{'objectid'}_$seg[0][$i]->{'segmentnumber'}.png")
	    {
		if (not -e "out/representative_frames/$seg[0][$i]->{'objectid'}/$seg[0][$i]->{'objectid'}_$seg[0][$i-1]->{'segmentnumber'}.png")
		{
		`cp out/representative_frames/$seg[0][$i]->{'objectid'}/$seg[0][$i]->{'objectid'}_$seg[0][$i]->{'segmentnumber'}.png out/representative_frames/$seg[0][$i]->{'objectid'}/$seg[0][$i]->{'objectid'}_$seg[0][$i-1]->{'segmentnumber'}.png`;
		`cp thumbnails/$seg[0][$i]->{'objectid'}/$seg[0][$i]->{'objectid'}_$seg[0][$i]->{'segmentnumber'}.png thumbnails/$seg[0][$i]->{'objectid'}/$seg[0][$i]->{'objectid'}_$seg[0][$i-1]->{'segmentnumber'}.png`;
		}
	    }
	}
}

